<?php

namespace Pronamic\WordPress\Pay\Extensions\Charitable;

use Charitable_Donation;
use Charitable_Donation_Processor;
use Charitable_Gateway;
use Pronamic\WordPress\Money\Parser as MoneyParser;
use Pronamic\WordPress\Pay\Payments\PaymentData as Pay_PaymentData;
use Pronamic\WordPress\Pay\Plugin;
use Pronamic\WordPress\Pay\Payments\Item;
use Pronamic\WordPress\Pay\Payments\Items;
use Pronamic\WordPress\Pay\Subscriptions\Subscription;
use Pronamic\WordPress\Pay\Subscriptions\SubscriptionInterval;
use Pronamic\WordPress\Pay\Subscriptions\SubscriptionPhase;
use Pronamic\WordPress\Pay\Core\Util;

/**
 * Title: WordPress pay Charitable payment data
 * Description:
 * Copyright: 2005-2020 Pronamic
 * Company: Pronamic
 *
 * @author  Remco Tolsma
 * @version 2.1.2
 * @since   1.0.0
 */
class PaymentData extends Pay_PaymentData {
	/**
	 * The donation ID
	 *
	 * @var int
	 */
	private $donation_id;

	/**
	 * Processor
	 *
	 * @var Charitable_Donation_Processor
	 */
	private $processor;

	/**
	 * Gateway
	 *
	 * @var Charitable_Gateway
	 */
	private $gateway;

	/**
	 * User data
	 *
	 * @var array
	 */
	private $user_data;

	/**
	 * Constructs and initializes an Charitable payment data object.
	 *
	 * @param int                           $donation_id Donation ID.
	 * @param Charitable_Donation_Processor $processor   Charitable donation processor.
	 * @param Charitable_Gateway            $gateway     Charitable gateway.
	 */
	public function __construct( $donation_id, $processor, $gateway ) {
		parent::__construct();

		$this->donation_id = $donation_id;
		$this->processor   = $processor;
		$this->gateway     = $gateway;

		$this->user_data = $processor->get_donation_data_value( 'user' );
	}

	/**
	 * Get source indicator
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_source()
	 * @return string
	 */
	public function get_source() {
		return 'charitable';
	}

	/**
	 * Get source ID.
	 *
	 * @return string
	 */
	public function get_source_id() {
		return \strval( $this->donation_id );
	}

	/**
	 * Get title.
	 *
	 * @return string
	 */
	public function get_title() {
		/* translators: %s: order id */
		return sprintf( __( 'Charitable donation %s', 'pronamic_ideal' ), $this->get_order_id() );
	}

	/**
	 * Get description
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_description()
	 * @return string
	 */
	public function get_description() {
		$description = $this->gateway->get_value( 'transaction_description' );

		if ( '' === $description ) {
			$description = $this->get_title();
		}

		// Replacements.
		$replacements = array(
			'{donation_id}' => $this->get_order_id(),
		);

		return strtr( $description, $replacements );
	}

	/**
	 * Get order ID
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_order_id()
	 * @return string
	 */
	public function get_order_id() {
		return \strval( $this->donation_id );
	}

	/**
	 * Get items
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_items()
	 * @return Items
	 */
	public function get_items() {
		$donation = new Charitable_Donation( $this->donation_id );

		// Items.
		$items = new Items();

		// Item
		// We only add one total item, because iDEAL cant work with negative price items (discount).
		$item = new Item();
		$item->set_number( $this->get_order_id() );
		$item->set_description( $this->get_description() );
		$item->set_quantity( 1 );

		// Price.
		$money_parser = new MoneyParser();

		$price = $money_parser->parse( $donation->get_total_donation_amount() )->get_value();

		$item->set_price( $price );

		$items->add_item( $item );

		return $items;
	}

	/**
	 * Get currency
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_currency_alphabetic_code()
	 * @return string
	 */
	public function get_currency_alphabetic_code() {
		return charitable_get_currency();
	}

	/**
	 * Get email.
	 *
	 * @return string|null
	 */
	public function get_email() {
		if ( \array_key_exists( 'email', $this->user_data ) ) {
			return $this->user_data['email'];
		}

		return null;
	}

	/**
	 * Get first name.
	 *
	 * @return string|null
	 */
	public function get_first_name() {
		if ( \array_key_exists( 'first_name', $this->user_data ) ) {
			return $this->user_data['first_name'];
		}

		return null;
	}

	/**
	 * Get last name.
	 *
	 * @return string|null
	 */
	public function get_last_name() {
		if ( \array_key_exists( 'last_name', $this->user_data ) ) {
			return $this->user_data['last_name'];
		}

		return null;
	}

	/**
	 * Get address.
	 *
	 * @return string|null
	 */
	public function get_address() {
		if ( \array_key_exists( 'address', $this->user_data ) ) {
			return $this->user_data['address'];
		}

		return null;
	}

	/**
	 * Get city.
	 *
	 * @return string|null
	 */
	public function get_city() {
		if ( \array_key_exists( 'city', $this->user_data ) ) {
			return $this->user_data['city'];
		}

		return null;
	}

	/**
	 * Get ZIP.
	 *
	 * @return string|null
	 */
	public function get_zip() {
		if ( \array_key_exists( 'postcode', $this->user_data ) ) {
			return $this->user_data['postcode'];
		}

		return null;
	}

	/**
	 * Get telephone number.
	 *
	 * @return string|null
	 */
	public function get_telephone_number() {
		if ( \array_key_exists( 'phone', $this->user_data ) ) {
			return $this->user_data['phone'];
		}

		return null;
	}

	/**
	 * Get normal return URL.
	 *
	 * @link https://github.com/woothemes/woocommerce/blob/v2.1.3/includes/abstracts/abstract-wc-payment-gateway.php#L52
	 * @return string
	 */
	public function get_normal_return_url() {
		return charitable_get_permalink( 'donation_receipt_page', array( 'donation_id' => $this->donation_id ) );
	}

	/**
	 * Get cancel URL.
	 *
	 * @return string
	 */
	public function get_cancel_url() {
		$cancel_url = charitable_get_permalink( 'donation_cancel_page', array( 'donation_id' => $this->donation_id ) );

		if ( ! $cancel_url ) {
			$cancel_url = esc_url_raw(
				add_query_arg(
					array(
						'donation_id' => $this->donation_id,
						'cancel'      => true,
					),
					wp_get_referer()
				)
			);
		}

		return $cancel_url;
	}

	/**
	 * Get success URL.
	 *
	 * @return string
	 */
	public function get_success_url() {
		return charitable_get_permalink( 'donation_receipt_page', array( 'donation_id' => $this->donation_id ) );
	}

	/**
	 * Get error URL.
	 *
	 * @return string
	 */
	public function get_error_url() {
		$cancel_url = charitable_get_permalink( 'donation_cancel_page', array( 'donation_id' => $this->donation_id ) );

		if ( ! $cancel_url ) {
			$cancel_url = esc_url_raw(
				add_query_arg(
					array(
						'donation_id' => $this->donation_id,
						'cancel'      => true,
					),
					wp_get_referer()
				)
			);
		}

		return $cancel_url;
	}

	/**
	 * Get subscription source ID.
	 *
	 * @return string|int|null
	 */
	public function get_subscription_source_id() {
		//return recurring_donation_id
		return $this->processor->get_donation_data_value( 'donation_plan' );
	}

	/**
	 * Get subscription.
	 *
	 * @return Subscription|null
	 */
	public function get_subscription() {
		if ( ! \class_exists( '\Charitable_Recurring' ) ) {
			return null;
		}

		$processor = $this->processor;

		// Monthly, yearly etc
		$donation_period = $processor->get_donation_data_value( 'donation_period', false );

		$recurring_id = $this->get_subscription_source_id();

		if ( empty( $donation_period ) || empty( $recurring_id ) ) {
			return null;
		}

		// Number of payments
		$donation_length = $processor->get_donation_data_value( 'donation_length', '' );
		$donation_length = empty( $donation_length ) ? '5200' : $donation_length;

		// duration between two $donation_period
		$donation_interval = $processor->get_donation_data_value( 'donation_interval', 1 );
		$recurring_key     = $processor->get_donation_data_value( 'recurring_donation_key' );
		$donation_key      = $processor->get_donation_data_value( 'donation_key' );

		// Convert period strings into  duration format.
		// link: https://www.php.net/manual/en/dateinterval.construct.php
		switch ( strtolower( $donation_period ) ) {
			case 'day':
				$donation_period = 'D';
				$donation_length = min( $donation_length, '36500' );
				break;
			case 'week':
				$donation_period = 'W';
				$donation_length = min( $donation_length, '5200' );
				break;
			case 'quarter':
				$donation_period   = 'M';
				$donation_length   = min( $donation_length, '400' );
				$donation_interval = $donation_interval * 3;
				break;
			case 'semiannual':
				$donation_period   = 'M';
				$donation_length   = min( $donation_length, '200' );
				$donation_interval = $donation_interval * 6;
				break;
			case 'month':
				$donation_period = 'M';
				$donation_length = min( $donation_length, '1200' );
				break;
			case 'year':
				$donation_period = 'Y';
				$donation_length = min( $donation_length, '100' );
				break;
			default:
				return null;
		}

		// Subscription.
		$subscription = new Subscription();

		$subscription->description = $this->get_description();

		// Phase.
		$phase = new SubscriptionPhase(
			$subscription,
			new \DateTimeImmutable(),
			new SubscriptionInterval( 'P' . $donation_interval . $donation_period ),
			$this->get_amount()
		);

		$phase->set_total_periods( intval( $donation_length ) );

		$subscription->add_phase( $phase );

		return $subscription;
	}
}
